<input 
    id="{{ id }}"
    name="{{ id }}-value"
    class="newsflash-input" 
    type="{{ type }}"
    value="{{ value }}"
    placeholder="{{ placeholder }}"
    autocomplete="off"
    {% if autofocus %}autofocus{% endif %}
    hx-post="{{ full_path }}"
    hx-trigger="input"
    hx-swap="none"
    hx-include="{{ hx_include|join(', ') }}"
    {% if hx_swap_oob %}hx-swap-oob="true"{% endif %}
>