<button
    id="{{ id }}"
    class="{{ classes|join(' ') }}"
    {% if disabled %}disabled{% endif %}
    hx-post="{{ full_path }}"
    hx-trigger="click"
    hx-swap="none"
    hx-include="{{ hx_include|join(', ') }}"
    {% if hx_swap_oob %}hx-swap-oob="true"{% endif %}
>
    {{ label }}
</button>