<textarea 
    id="{{ id }}"
    name="{{ id }}-value"
    class="newsflash-input newsflash-textarea" 
    rows="{{ rows }}"
    placeholder="{{ placeholder }}"
    spellcheck="{{ 'true' if spellcheck else 'false' }}"
    autocomplete="off"
    {% if autofocus %}autofocus{% endif %}
    hx-post="{{ full_path }}"
    hx-trigger="input delay:300ms"
    hx-swap="none"
    hx-include="{{ hx_include|join(', ') }}"
    {% if hx_swap_oob %}hx-swap-oob="true"{% endif %}
>{{ value }}</textarea>